@@ -24,6 +24,8 @@ from cubicweb.predicates import (
     ExpectedValuePredicate,
     match_form_params,
     match_http_method,
+    anonymous_user,
+    authenticated_user,
 )
 from cubicweb.uilib import exc_message
 from cubicweb.utils import json_dumps
@@ -33,6 +35,8 @@ from cubicweb.web.views.urlrewrite import rgx_action, SchemaBasedRewriter
 from cubicweb import Binary
 from cubicweb_rqlcontroller.rql_schema_holder import RqlIOSchemaHolder
 
+from cubicweb_rqlcontroller.predicates import match_all_http_headers
+
 
 ARGRE = re.compile(r"__r(?P<ref>\d+)$")
 DATARE = re.compile(r"__f(?P<ref>.+)$")
@@ -87,7 +91,7 @@ class RqlIOSchemaController(Controller):
         )
 
 
-class RqlIOController(Controller):
+class BaseRqlIOController(Controller):
     """posted rql queries and arguments use the following pattern:
 
     [('INSERT CWUser U: U login %(login)s, U upassword %(pw)s',
@@ -110,15 +114,9 @@ class RqlIOController(Controller):
 
     """
 
+    __abstract__ = True
     __regid__ = "rqlio"
-    __select__ = (
-        match_http_method("POST")
-        & match_request_content_type(
-            "application/json", "multipart/form-data", mode="any"
-        )
-        & match_form_params("version")
-    )
-    require_csrf = False
+    __select__ = match_http_method("POST") & match_form_params("version")
 
     def json(self):
         contenttype = self._cw.get_header("Content-Type", raw=False)
@@ -186,6 +184,49 @@ class RqlIOController(Controller):
         return output
 
 
+class JsonRqlIOController(BaseRqlIOController):
+    """RqlIOController with csrf desactivated for application/json because
+    application/json can't be sent through a form
+    """
+
+    __select__ = BaseRqlIOController.__select__ & match_request_content_type(
+        "application/json", mode="any"
+    )
+    require_csrf = False
+
+
+class MultipartRqlIOController(BaseRqlIOController):
+    """RqlIOController with csrf activated for cookie authenticated user
+    and multipart/form-data.
+
+    To have csrf deactivated with multipart/form-data, install
+    cubicweb-signedrequest and use an authentication method.
+    """
+
+    __select__ = (
+        BaseRqlIOController.__select__
+        & ~match_all_http_headers("Authorization")  # no Authorization == cookie
+        & match_request_content_type("multipart/form-data", mode="any")
+        & authenticated_user()
+    )
+
+
+class AnonMultipartRqlIOController(BaseRqlIOController):
+    """RqlIOController with csrf desactivated for anonymous_user.
+    This allows public usage of the route.
+
+    It is expected anonymous user should have only read auhorization.
+    """
+
+    __select__ = (
+        BaseRqlIOController.__select__
+        & ~match_all_http_headers("Authorization")
+        & match_request_content_type("multipart/form-data", mode="any")
+        & anonymous_user()
+    )
+    require_csrf = False
+
+
 class RQLIORewriter(SchemaBasedRewriter):
     rules = [
         (re.compile("/rqlio/schema"), rgx_action(controller="rqlio_schema")),
